@@ -27,7 +27,9 @@
 
 // Test dictionary -> double elements -> dictionary elements round trip
 
-// Flags: --allow-natives-syntax --unbox-double-arrays --expose-gc --noparallel-recompilation
+// Flags: --allow-natives-syntax --unbox-double-arrays --expose-gc
+// Flags: --noparallel-recompilation
+
 var large_array_size = 100000;
 var approx_dict_to_elements_threshold = 70000;
 
@@ -350,6 +352,9 @@ function testOneArrayType(allocator) {
   assertTrue(%HasFastDoubleElements(large_array));
 }
 
+// Force gc here to start with a clean heap if we repeat this test multiple
+// times.
+gc();
 testOneArrayType(make_object_like_array);
 testOneArrayType(Array);
 
